use std::any::Any;
use std::path::{Path, Component};
use std::io::{Read, Result};
use std::collections::HashMap;
use std::collections::hash_map::Values;
use libarchive::archive::{ReadFilter, ReadFormat, Entry, FileType};
use libarchive::reader::{Builder, Reader};
use package::Package;

pub struct Repo {
    cache: HashMap<String, Package>,
}

fn parse_entry(name: &str) -> Option<(&str, &str)> {
    name.rfind("-").and_then(|dash1| {
        name[..dash1].rfind("-").map(|dash2| {
            (&name[..dash2], &name[dash2 + 1..])
        })
    })
}

fn get_pkg_metadata(path: &str) -> Option<(&str, &str)> {
    let components: Vec<_> = Path::new(path)
        .components()
        .collect();

    match components[..] {
        [Component::Normal(pkg), Component::Normal(_)] => pkg.to_str().and_then(parse_entry),
        _ => None,
    }
}

impl Repo {
    pub fn new() -> Self {
        Repo { cache: HashMap::new() }
    }

    pub fn load<T: Any + Read>(&mut self, src: T) -> Result<()> {
        let mut builder = Builder::new();
        builder.support_format(ReadFormat::All).unwrap();
        builder.support_filter(ReadFilter::All).unwrap();

        let mut reader = builder.open_stream(src).unwrap();
        loop {
            let pkg = {
                let entry = reader.next_header();
                if entry.is_none() {
                    break;
                }

                entry.and_then(|entry| match entry.filetype() {
                        FileType::RegularFile => get_pkg_metadata(entry.pathname()),
                        _ => None,
                    })
                    .map(|(pkgname, pkgver)| {
                        self.cache
                            .entry(pkgname.into())
                            .or_insert(Package::new(pkgname, pkgver))
                    })
            };

            pkg.and_then(|pkg| {
                reader.read_block() // TODO: loop and error handling
                    .ok()
                    .unwrap()
                    .and_then(|block| pkg.read_desc(block).ok())
            });
        }
        Ok(())
    }

    pub fn pkgs(&self) -> Values<String, Package> {
        self.cache.values()
    }
}
