#![feature(slice_patterns)]

extern crate base64;
extern crate getopts;
extern crate libarchive;
extern crate libc;
#[macro_use]
extern crate nom;
extern crate rand;
extern crate uname;

mod storage;
mod database;
mod desc;
mod package;
mod elephant;

use std::env;
use std::io::Result;
use getopts::Options;
use uname::uname;
use storage::Storage;
use elephant::elephant;
use database::Repo;

fn system_arch() -> Result<String> {
    uname().map(|uts| uts.machine)
}

fn print_usage(program: &str, opts: Options) {
    let brief = format!("usage: {} [options] <database> [pkgs ...]", program);
    print!("{}", opts.usage(&brief));
}

fn load_repo(name: &str, root: &Storage) -> Result<Repo> {
    let mut repo = Repo::new();
    root.open(&format!("{}.db", name)).and_then(|db| repo.load(db))?;
    root.open(&format!("{}.files", name)).and_then(|files| repo.load(files))?;
    Ok(repo)
}

fn main() {
    let args: Vec<String> = env::args().collect();
    let program = args[0].clone();

    let mut opts = Options::new();
    opts.optflag("h", "help", "display this help and exit");
    opts.optflag("V", "version", "display version");
    opts.optflag("v", "verbose", "verbose output");
    opts.optflag("f", "files", "build the files database");
    opts.optflag("l", "list", "list packages in the repository");
    opts.optopt("r", "root", "set the root for the repository", "PATH");
    opts.optopt("p", "pool", "set the pool to find packages in", "PATH");
    opts.optopt("m", "arch", "the architecture of the database", "ARCH");
    opts.optflag("s", "sign", "create a database signature");
    opts.optflag("j", "bzip2", "filter the archive through bzip2");
    opts.optflag("J", "xz", "filter the archive through xz");
    opts.optflag("z", "gzip", "filter the archive through gzip");
    opts.optflag("Z", "compress", "filter the archive through compress");
    opts.optflag("", "reflink", "use reflinks instead of symlinks");
    opts.optflag("", "rebuild", "force rebuild the repository");
    opts.optflag("", "elephant", "print an elephant");

    let matches = match opts.parse(&args[1..]) {
        Ok(m) => m,
        Err(f) => {
            print!("{}\n", f);
            std::process::exit(1);
        }
    };

    if matches.opt_present("help") {
        print_usage(&program, opts);
    } else if matches.opt_present("elephant") {
        println!("{}", elephant().unwrap());
    } else {
        let root_dir = matches.opt_str("root").unwrap_or(".".into());

        let repo = Storage::new(&root_dir)
            .and_then(|root| load_repo("vodik", &root))
            .unwrap();

        if matches.opt_present("list") {
            let mut pkgs: Vec<_> = repo.pkgs().collect();
            pkgs.sort_by(|&a, &b| a.name.cmp(&b.name));

            for pkg in pkgs {
                // println!("{} {}", pkg.name, pkg.version);
                println!("{} {}: {:#?}", pkg.name, pkg.version, pkg);
            }
        }
    }
}
