use std::collections::HashMap;

#[derive(Clone,Hash,PartialEq,Eq,Debug)]
pub enum Entry {
    Filename,
    Name,
    Base,
    Version,
    Description,
    Groups,
    PackageSize,
    InstallSize,
    SHA256Sum,
    PGPSig,
    Url,
    License,
    Arch,
    BuildDate,
    Packager,
    Replaces,
    Depends,
    Conflicts,
    Provides,
    OptDepends,
    MakeDepends,
    CheckDepends,
    Files,
    Deltas,
}

#[derive(Debug)]
pub enum Arch {
    Any,
    Arch(String),
    Unknown,
}

impl Default for Arch {
    fn default() -> Self {
        Arch::Unknown
    }
}

impl<'a> From<&'a str> for Arch {
    fn from(arch: &str) -> Self {
        match arch {
            "any" => Arch::Any,
            _ => Arch::Arch(arch.into()),
        }
    }
}

#[derive(Debug,PartialEq)]
pub enum Metadata {
    Text(String),
    Size(u64),
    Timestamp(i64),
    List(Vec<String>),
}

// TODO this is not quite appropriate
impl<'a> From<(&'a Entry, &'a Vec<&'a str>)> for Metadata {
    fn from((entry, values): (&'a Entry, &'a Vec<&'a str>)) -> Self {
        fn single_value(values: &[&str]) -> Metadata {
            Metadata::Text(values[0].into())
        }

        fn vec_value(values: &[&str]) -> Metadata {
            Metadata::List(values.iter().map(|&x| x.to_owned()).collect())
        }

        fn u64_parse(values: &[&str]) -> Metadata {
            Metadata::Size(values[0].parse().unwrap())
        }

        fn i64_parse(values: &[&str]) -> Metadata {
            Metadata::Timestamp(values[0].parse().unwrap())
        }

        match *entry {
            Entry::Filename => single_value(values),
            Entry::Base => single_value(values),
            Entry::Description => single_value(values),
            Entry::Groups => vec_value(values),
            Entry::PackageSize => u64_parse(values),
            Entry::InstallSize => u64_parse(values),
            Entry::SHA256Sum => single_value(values),
            Entry::PGPSig => single_value(values),
            Entry::Url => single_value(values),
            Entry::License => vec_value(values),
            Entry::BuildDate => i64_parse(values),
            Entry::Packager => single_value(values),
            Entry::Replaces => vec_value(values),
            Entry::Depends => vec_value(values),
            Entry::Conflicts => vec_value(values),
            Entry::Provides => vec_value(values),
            Entry::OptDepends => vec_value(values),
            Entry::MakeDepends => vec_value(values),
            Entry::CheckDepends => vec_value(values),
            Entry::Files => vec_value(values),
            Entry::Deltas => vec_value(values),

            // TODO: Not all entries are extended metadata
            _ => panic!("Unhandled metadata")
        }
    }
}

#[derive(Debug)]
pub struct Package {
    pub name: String,
    pub version: String,
    pub arch: Arch,
    pub metadata: HashMap<Entry, Metadata>,
}

impl Package {
    pub fn new(pkgname: &str, pkgver: &str) -> Self {
        Package {
            name: pkgname.into(),
            version: pkgver.into(),
            arch: Default::default(),
            metadata: HashMap::new(),
        }
    }
}
